@@ -14,14 +14,16 @@ package Badger::Class::Config;
 
 use Carp;
 use Badger::Debug ':dump';
+use Badger::Config::Schema;
 use Badger::Class
     version   => 0.01,
     debug     => 0,
     base      => 'Badger::Exporter Badger::Base',
     import    => 'class CLASS',
-    words     => 'CONFIG',
+    words     => 'CONFIG_SCHEMA CONFIG_ITEMS',
     constants => 'HASH ARRAY DELIMITER',
     constant  => {
+        SCHEMA        => 'Badger::Config::Schema',
         CONFIG_METHOD => 'configure',
         VALUE         => 1,
         NOTHING       => 0,
@@ -35,13 +37,31 @@ use Badger::Class
 sub export {
     my $class  = shift;
     my $target = shift;
-    my $schema = $class->schema(@_);
+    $class->debug("export to $target: ", join(', ', @_)) if DEBUG;
+    my $params = @_ == 1 ? shift : { @_ };
+    my $schema = $class->schema($target, $params);
+    my $items  = $schema->items;
+    
+    $class->debug(
+        "exporting CONFIG_SCHEMA to $target: $schema"
+    ) if DEBUG;
     
     $class->export_symbol(
         $target,
-        CONFIG,
+        CONFIG_SCHEMA,
         \$schema
     );
+
+    $class->debug(
+        "export CONFIG_ITEMS to $target: ", 
+        $class->dump_data($items)
+    ) if DEBUG;
+    
+    $class->export_symbol(
+        $target,
+        CONFIG_ITEMS,
+        \$items,
+    );
     
     $class->export_symbol(
         $target, 
@@ -50,78 +70,34 @@ sub export {
     );
 }
 
-
 sub schema {
     my $class  = shift;
-    my $config = @_ == 1 ? shift : { @_ };
-    my ($name, $info, @aka, $fallback, $test, @schema);
+    my $target = shift;
+    my $config = @_ == 1 ? (ref $_[0] eq ARRAY ? [@{$_[0]}] : shift) : [ @_ ];
 
     $class->debug("Generating schema from config: ", $class->dump_data($config))
         if DEBUG;
     
     $config = [ split(DELIMITER, $config) ]
         unless ref $config;
+
+    # inherit any other items define in base classes
+    my $items = class($target)->list_vars(CONFIG_ITEMS);
     
-    $config = { 
-        map { 
-            ref($_) eq HASH 
-                ? ($_->{ name } => $_)
-                : ($_           => { })
-        } @$config 
-    } if ref $config eq ARRAY;
-
-    $class->debug("Canonical config: ", $class->dump_data($config))
-        if DEBUG;
-        
-    while (($name, $info) = each %$config) {
-        if (ref $info eq HASH) {
-            # ok
-        }
-        else {
-            $info = { default => $info };
-        }
-        
-        $info->{ required } = 1 
-            if $name =~ s/!$//;
-            
-        $info->{ default } = $1
-            if $name =~ s/=(\S+)$//;
-
-        # name can be 'name|alias1|alias2|...'
-        ($name, @aka) = split(/\|/, $name);
-        
-        # $info is now a hash ref
-        $info->{ name } = $name
-            unless defined $info->{ name };
-
-        # aliases can be specified as a list ref or string which we split
-        $fallback = $info->{ fallback } || [];
-        $fallback = [ split(DELIMITER, $fallback) ]
-            unless ref $fallback eq ARRAY;
-        push(@$fallback, @aka);
-
-        foreach my $item (@$fallback) {
-            next unless $item =~ /:/;
-            my ($type, $data) = split(/:/, $item, 2);
-            my $code = $class->can('configure_' . $type)
-                || return $class->error_msg( bad_type => $name, $type );
-            $item = [ $code, $data ];
-        }
-        
-        # add any aliases specified as part of the name and bind them 
-        # back into the field info hash
-        $info->{ fallback } = $fallback;
-
-        $class->debug("Adding config schema element for $name: ", $class->dump_data($info)) if DEBUG;
-
-        push(@schema, $info);
-    }
-
-#    $class->debug('schema: ', $class->dump_data_inline(\@schema)) if DEBUG;
-    
-    return \@schema;
+    $class->SCHEMA->new(
+        class    => $target,
+        schema   => $config,
+        fallback => $class,
+        extend   => $items,
+    );
 }
-    
+
+sub fallback {
+    my ($self, $name, $type, $data) = @_;
+    my $code = $self->can('configure_' . $type) || return;
+    return [ $code, $data ];
+}
+
 
 #-----------------------------------------------------------------------
 # this method is mixed into the target module
@@ -130,68 +106,35 @@ sub schema {
 sub configure {
     my ($self, $config, $target) = @_;
     my $class  = class($self);
-    my $schema = $class->list_vars(CONFIG);
-    my ($element, $name, $alias, $code, @args, $ok, $value);
-    
-    # if a specific $target isn't defined then we default to updating $self
-    $target ||= $self;
+    my $schema = $class->any_var(CONFIG_SCHEMA);
 
-    $self->debug("configure(", CLASS->dump_data_inline($config), ')') if DEBUG;
-    $self->debug("schema: ", CLASS->dump_data($schema)) if DEBUG;
-    
-    ELEMENT: foreach $element (@$schema) {
-        $name = $element->{ name };
-        
-        FALLBACK: foreach $alias ($name, @{ $element->{ fallback } }) {
-            if (ref $alias) {
-                $self->debug("Dispatching handler to set $name\n") if DEBUG;
-                ($code, @args) = @$alias;
-                ($ok, $value) = $code->($self, $class, $name, $config, @args);
-                if ($ok) {
-                    $target->{ $name } = $value;
-                    next ELEMENT;
-                }
-            }
-            elsif (defined $config->{ $alias }) {
-                $self->debug("Looking for $alias in config to set $name\n") if DEBUG;
-                $target->{ $name } = $config->{ $alias };
-                next ELEMENT;
-            }
-            else {
-                $self->debug("Nothing found for $alias to set $name\n") if DEBUG;
-            }
-        }
-        
-        if (exists $element->{ default }) {
-            $target->{ $name } = $element->{ default };
-            next ELEMENT;
-        }
-        
-        if ($element->{ required }) {
-            return $self->error_msg( $element->{ error } || missing => $name );
-        }
-    }
+    # if a specific $target isn't defined then we default to updating $self
+    $schema->configure($config, $target || $self, $self);
     
     return $self;
 }
-
+    
 
 #-----------------------------------------------------------------------
 # These handlers implement the various fallback types for providing 
 # configuration data.  The schema() method maps fallacks specified as
 # 'pkg:FOO' and 'class:BAR', for example, to the configure_pkg() and 
 # configure_class() handlers, passing the token following the colon as 
-# an argument.  They are called as code refs, but the object they're 
-# configuring is passed as the first argument, $self. So they look
-# like object methods, but they're not exported into the object's 
-# namespace.
+# an argument.  They are called as code refs, but the class of the 
+# object that they're configuring is passed as the first argument, $class. 
+# So they look like class methods, but they're not exported into the 
+# object's namespace.  The $target is usually the object that's being
+# configured, e.g. when $self->configure($config) is called, but it might
+# also be a bare hash, e.g. $target = { }; $self->configure($config, $target)
 #-----------------------------------------------------------------------
-    
-sub configure_pkg {
-    my ($self, $class, $name, $config, $var) = @_;
-    my $value = $class->var($var);
 
-    $self->debug(
+# TODO: move these into Badger::Config::Item or somewhere else
+
+sub configure_pkg {
+    my ($class, $name, $config, $target, $var) = @_;
+    my $value = class($class)->var($var);
+
+    $class->debug(
         "Looking for \$$var package variable in $class to set $name: ", 
         defined $value ? $value : '<undef>'
     ) if DEBUG;
@@ -201,12 +144,11 @@ sub configure_pkg {
         : (NOTHING);
 }
 
-
 sub configure_class {
-    my ($self, $class, $name, $config, $var) = @_;
-    my $value = $class->any_var_in( split(':', $var) );
+    my ($class, $name, $config, $target, $var) = @_;
+    my $value = class($class)->any_var_in( split(':', $var) );
 
-    $self->debug(
+    $class->debug(
         "Looking for \$$var class variable in $class to set $name: ", 
         defined $value ? $value : '<undef>'
     ) if DEBUG;
@@ -216,12 +158,11 @@ sub configure_class {
         : (NOTHING);
 }
 
-
 sub configure_env {
-    my ($self, $class, $name, $config, $var) = @_;
+    my ($class, $name, $config, $target, $var) = @_;
     my $value = $ENV{ $var };
 
-    $self->debug(
+    $class->debug(
         "Looking for $var environment variable to set $name: ",
         defined $value ? $value : '<undef>'
     ) if DEBUG;
@@ -232,18 +173,18 @@ sub configure_env {
 }
 
 sub configure_method {
-    my ($self, $class, $name, $config, $method) = @_;
+    my ($class, $name, $config, $target, $method) = @_;
 
     # see if the object has the required method - note we must call 
     # error_msg against CLASS (Badger::Class::Config) to use the 'bad_method'
     # message defined above.
-    my $code = $self->can($method)
-        || return CLASS->error_msg( bad_method => $class, $name, $method );
+    my $code = $class->can($method)
+        || return CLASS->error_msg( bad_method => class($class), $name, $method );
 
     # call the code and do the usual shuffle
-    my $value = $code->($self);
+    my $value = $code->($class);
 
-    $self->debug(
+    $class->debug(
         "Called $method() method to set $name: ",
         defined $value ? $value : '<undef>'
     ) if DEBUG;
@@ -253,6 +194,22 @@ sub configure_method {
         : (NOTHING);
 }
 
+sub configure_target {
+    my ($class, $name, $config, $target, $var) = @_;
+
+    my $value = $target->{ $var };
+    
+    $class->debug(
+        "Looking for $var in $class target $target to set $name: ", 
+        defined $value ? $value : '<undef>'
+    ) if DEBUG;
+
+    return defined $value
+        ? (VALUE => $value)
+        : (NOTHING);
+}
+
+
 
 1;
 
@@ -316,15 +273,15 @@ delimited string.
 More complex configurations can be specified using list and hash references,
 but we'll keep things simple for now.
 
-Using the module as shown here has two immediate effects.  The first is that
-the C<$CONFIG> package variable will be defined in C<Your::Module> containing
-a reference to the configuration schema for your module.  This schema contains
-information about the configuration items which in this example are C<foo>
-and C<bar>.  The second effect is to define a L<configure()> method which 
-uses this schema to configure your object using the configuration options 
-passed to the constructor method.  You can call this method from your 
-L<init()|Badger::Base/init()> method (if you're using L<Badger::Base>)
-or from your own construction or initialisation methods.
+Using the module as shown here has two immediate effects. The first is that
+the C<$CONFIG_SCHEMA> package variable will be defined in C<Your::Module>
+containing a reference to the configuration schema for your module. This
+schema contains information about the configuration items which in this
+example are C<foo> and C<bar>. The second effect is to define a L<configure()>
+method which uses this schema to configure your object using the configuration
+options passed to the constructor method. You can call this method from your
+L<init()|Badger::Base/init()> method (if you're using L<Badger::Base>) or from
+your own construction or initialisation methods.
 
     sub init {
         my ($self, $config) = @_;
@@ -362,6 +319,7 @@ call object methods.
         'bar|class:BAR',                # fallback to $BAR class var
         'baz|env:BAZ',                  # fallback to $BAZ environment var
         'bam|method:BAM';               # fallback to BAM() method
+        'wam|target:slam';              # fallback to $target->{ slam }
 
 Bear in mind that Perl implements constants using subroutines.  Thus, you
 can access a constant defined in a package/class by calling it as a
@@ -437,13 +395,13 @@ configuration items.
 =head2 schema()
 
 This method is used internally to define a configuration schema.  It exports
-it as the C<$CONFIG> package variable into the calling module.
+it as the C<$CONFIG_SCHEMA> package variable into the calling module.
 
 =head2 configure($config,$target)
 
 This method is exported the calling module to perform the configuration
-process. It used the configuration schema stored in the C<$CONFIG> package
-variable by the L<schema()> method.  It is typically called from a
+process. It used the configuration schema stored in the C<$CONFIG_SCHEMA>
+package variable by the L<schema()> method. It is typically called from a
 construction or initialisation method.
 
 The first argument should be a reference to a hash array of configuration
@@ -479,13 +437,18 @@ configuration options.
 This method is used internally to call object methods to return default
 configuration values.
 
+=head2 configure_target()
+
+This method is used internally to look inside the target object or hash array
+to return default configuration values.
+
 =head1 AUTHOR
 
 Andy Wardley L<http://wardley.org/>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2008 Andy Wardley.  All Rights Reserved.
+Copyright (C) 2008-2009 Andy Wardley.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
