@@ -43,6 +43,7 @@ use constant {
     LAST            => -1,
     CRLF            => "\015\012",          # unambiguous CR+LF sequence
     PKG             => '::',                # for joining pkg names
+    DOT             => '.',                 # for joining dotty names
     REFS            => 'refs',              # no strict REFS
     ONCE            => 'once',              # use warnings ONCE
     WARN            => 'warn',              # default warning handler
@@ -50,11 +51,10 @@ use constant {
     SPACE           => ' ',                 # and the answer is none, none more blank
     DELIMITER       => qr/(?:,\s*)|\s+/,    # match a comma or whitespace
     WILDCARD        => qr/[\*\?]/,          # wildcards: *.html foo??.txt
-
 };
 
 CONSTANTS->export_any(qw( 
-    CONSTANTS FIRST LAST CRLF PKG REFS ONCE WARN BLANK SPACE 
+    CONSTANTS FIRST LAST CRLF PKG DOT REFS ONCE WARN BLANK SPACE 
     DELIMITER WILDCARD 
 ));
 
@@ -204,6 +204,10 @@ used to construct symbol references.
     
     my $var = ${ $pkg.PKG.EXAMPLE };   # same as: ${"${pkg}::EXAMPLE"}
 
+=head2 DOT
+
+An alias for a dot C<.>.
+
 =head2 REFS
 
 The literal string C<refs>.  Typically used like so:
@@ -265,7 +269,7 @@ Andy Wardley L<http://wardley.org/>
 
 =head1 COPYRIGHT
 
-Copyright (C) 1996-2008 Andy Wardley.  All Rights Reserved.
+Copyright (C) 1996-2009 Andy Wardley.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
