@@ -15,20 +15,20 @@ package Badger::Debug;
 use Carp;
 use Badger::Rainbow 
     ANSI => 'bold red yellow green cyan white';
-use Scalar::Util 'blessed';
+use Scalar::Util qw( blessed refaddr );
 use Badger::Class
     base      => 'Badger::Exporter',
     version   => 0.01,
     constants => 'PKG REFS SCALAR ARRAY HASH CODE REGEX DELIMITER',
     words     => 'DEBUG',
     import    => 'class',
-#    utils     => 'blessed reftype',
     constant  => {
         UNDEF => '<undef>',
     },
     exports   => {
         tags  => {
-            debug => 'debugging debug debug_up debug_caller debug_args',
+            debug => 'debugging debug debugf debug_up debug_at debug_caller 
+                      debug_callers debug_args',
             dump  => 'dump dump_data dump_data_inline
                       dump_ref dump_hash dump_list dump_text'
         },
@@ -46,9 +46,14 @@ use Badger::Class
 our $PAD       = '    ';
 our $MAX_TEXT  = 48;
 our $MAX_DEPTH = 3;     # prevent runaways in debug/dump
-our $FORMAT    = "[<class> line <line>] <msg>"  
+our $FORMAT    = "[<where> line <line>]\n<msg>"  
     unless defined $FORMAT;
-our $CALLER_UP = 0;     # hackola to allow debug() to use a different caller
+our $PROMPT    = '> ' 
+    unless defined $PROMPT;
+our $MESSAGE   = "$PROMPT%s";
+our $CALLER_UP = 0;      # hackola to allow debug() to use a different caller
+our $CALLER_AT = { };    # ditto
+our $DUMPING   = { };
 our $DEBUG     = 0 unless defined $DEBUG;
 
 
@@ -61,6 +66,7 @@ sub _export_debug_dumps {
     $self->export_symbol($target, dumper => sub {
         $_[0]->dump_hash($_[0],$_[1],$value);
     });
+    unshift(@$symbols, ':dump');
     return $self;
 }
 
@@ -149,21 +155,51 @@ sub debug {
     my $self   = shift;
     my $msg    = join('', @_),
     my $class  = ref $self || $self;
-    my $format = $FORMAT;
+    my $format = $CALLER_AT->{ format } || $FORMAT;
     my ($pkg, $file, $line) = caller($CALLER_UP);
-    $class .= " ($pkg)" unless $class eq $pkg;
-    $msg .= "\n" unless $msg =~ /\n$/;
+    my (undef, undef, undef, $sub) = caller($CALLER_UP + 1);
+    if (defined $sub) {
+        $sub =~ s/.*?([^:]+)$/::$1()/;
+    }
+    else {
+        $sub = '';
+    }
+    my $where  = ($class eq $pkg) 
+        ? $class . $sub
+        : $pkg   . $sub . " ($class)";
+
+    $msg = join("\n", map { sprintf($MESSAGE, $_) } split("\n", $msg));
+#    $msg =~ s/^/$PROMPT/gm;
+
+    # We load this dynamically because it uses Badger::Debug and we don't
+    # want to end up in a gruesome birth spiral 
+    require Badger::Timestamp;
+    my $now  = Badger::Timestamp->now;
     my $data = {
         msg   => $msg,
+        where => $where,
         class => $class,
         file  => $file,
         line  => $line,
+        pkg   => $pkg,
+        sub   => $sub,
+        date  => $now->date,
+        time  => $now->time,
+        %$CALLER_AT,
     };
-    $format =~ s/<(\w+)>/defined $data->{ $1 } ? $data->{ $1 } : "<$1 undef>"/eg;
+    $format  =~ s/<(\w+)>/defined $data->{ $1 } ? $data->{ $1 } : "<$1 undef>"/eg;
+    $format .= "\n" unless $format =~ /\n$/;
+    
     print STDERR $format;
 }
 
 
+sub debugf {
+    local $CALLER_UP = 1;
+    shift->debug( sprintf(shift, @_) );
+}
+
+
 sub debug_up {
     my $self = shift;
     local $CALLER_UP = shift;
@@ -171,6 +207,14 @@ sub debug_up {
 }
 
 
+sub debug_at {
+    my $self = shift;
+    local $CALLER_AT = shift;
+    local $CALLER_UP = 1;
+    $self->debug(@_);
+}
+
+
 sub debug_caller {
     my $self = shift;
     my ($pkg, $file, $line, $sub) = caller(1);
@@ -181,6 +225,24 @@ sub debug_caller {
 }
 
 
+sub debug_callers {
+    my $self = shift;
+    my $msg  = '';
+    my $i    = 1;
+    
+    while (1) {
+        my @info = caller($i);
+        last unless @info;
+        my ($pkg, $file, $line, $sub) = @info;
+        $msg .= sprintf(
+            "%4s: Called from %s in %s at line %s\n",
+            '#' . $i++, $sub, $file, $line
+        );
+    }
+    $self->debug($msg);
+}
+
+
 sub debug_args {
     my $self = shift;
     $self->debug_up( 
@@ -222,6 +284,12 @@ sub dump {
 
 
 sub dump_data {
+    local $DUMPING = { };
+    _dump_data(@_);
+}
+
+    
+sub _dump_data {
     if (! defined $_[1]) {
         return UNDEF;
     }
@@ -240,6 +308,7 @@ sub dump_data {
 
 sub dump_ref {
     my ($self, $data, $indent) = @_;
+    return "<$data>" if $DUMPING->{ $data }++;
     
     # TODO: change these to reftype
     if (UNIVERSAL::isa($data, HASH)) {
@@ -289,7 +358,7 @@ sub dump_hash {
     
     return "\{\n" 
         . join( ",\n", 
-                map { "$pad$PAD$_ => " . $self->dump_data($hash->{$_}, $indent + 1) }
+                map { "$pad$PAD$_ => " . _dump_data($self, $hash->{$_}, $indent + 1) }
                 sort grep { $keys ? $keys->{ $_ } : 1 } keys %$hash 
            ) 
         . "\n$pad}";
@@ -304,7 +373,7 @@ sub dump_list {
     return '[ ]' unless @$list;
     return "\[\n$pad$PAD" 
         . ( @$list 
-            ? join(",\n$pad$PAD", map { $self->dump_data($_, $indent + 1) } @$list) 
+            ? join(",\n$pad$PAD", map { _dump_data($self, $_, $indent + 1) } @$list) 
             : '' )
         . "\n$pad]";
 }
@@ -339,9 +408,10 @@ sub enable_colour {
     print bold green "Enabling debug in $symbol from $target\n";
 
     # colour the debug format
+    $MESSAGE = cyan($PROMPT) . yellow('%s');
     $FORMAT 
-         = cyan('[<class> line <line>]')
-         . yellow(' <msg>');
+         = cyan('[<where> line <line>]')
+         . "\n<msg>";
 
     # exceptions are in red
     $Badger::Exception::FORMAT 
@@ -748,7 +818,7 @@ of this in use.
 =head2 :debug
 
 Imports all of the L<debug()>, L<debugging()>, L<debug_up()>, 
-L<debug_caller()> and L<debug_args()> methods.
+L<debug_caller()>, L<debug_callers> and L<debug_args()> methods.
 
 =head2 :dump
 
@@ -772,6 +842,12 @@ was called.
 At some point in the future this will be extended to allow you to tie in
 debug hooks, e.g. to forward to a logging module.
 
+=head2 debugf($format, $arg1, $arg2, ...)
+
+This method provides a C<printf()>-like wrapper around L<debug()>.
+
+    $object->debugf('%s is %s', e => 2.718);    # e is 2.718
+
 =head2 debug_up($n, $msg1, $msg2, ...)
 
 The L<debug()> method generates a message showing the file and line number
@@ -802,6 +878,47 @@ case if we called C<debug(...)> or C<debug_up(1,...)>), it will correctly
 reporting the line number of the call to C<trace()> in the C<parse()> 
 method.
 
+=head2 debug_at($info, $message)
+
+This method is a wrapper around L<debug()> that allows you to specify a
+different location to be added to the message generated.
+
+    $at->debug_at(
+        { 
+            where => 'At the edge of time', 
+            line  => 420 
+        }, 
+        'Flying sideways'
+    );
+
+This generates the following debug message:
+
+    [At the edge of time line 420] Flying sideways
+
+Far out, man!
+
+You can change the L<$FORMAT> package variable to define a different message
+structure.  As well as the pre-defined placeholders (see the L<$FORMAT> 
+documentation) you can also define your own custom placeholders like
+C<E<lt>serverE<gt>> in the following example.
+
+    $Badger::Debug::FORMAT = '<server>: <msg> at line <line> of <file>';
+
+You must then provide values for the additional placeholder in the C<$info>
+hash array when you call the L<debug_at()> method.
+
+    $at->debug_at(
+        { server => 'Alpha' },
+        'Normality is resumed'
+    );
+
+You can also specify a custom format in the C<$info> hash array.
+
+    $at->debug_at(
+        { format => '<msg> at line <line> of <file>' }, 
+        'Normality is resumed'
+    );
+
 =head2 debug_caller()
 
 Prints debugging information about the current caller.
@@ -811,6 +928,15 @@ Prints debugging information about the current caller.
         $self->debug_caller;
     }
 
+=head2 debug_callers()
+
+Prints debugging information about the complete call stack.
+
+    sub wibble {
+        my $self = shift;
+        $self->debug_callers;
+    }
+
 =head2 debug_args()
 
 Prints debugging information about the arguments passed.
@@ -929,11 +1055,30 @@ Enables colourful debugging and error messages.
 The L<debug()> method uses the message format in the C<$FORMAT>
 package variable to generate debugging messages.  The default value is:
 
-    [<class> line <line>] <msg>
+    [<where> line <line>] <msg>
 
-The C<E<lt>classE<gt>>, C<E<lt>lineE<gt>> and C<E<lt>msgE<gt>> markers
+The C<E<lt>where<gt>>, C<E<lt>lineE<gt>> and C<E<lt>msgE<gt>> markers
 denote the positions where the class name, line number and debugging 
-message are inserted.
+message are inserted.  You can embed any of the following placeholders 
+into the message format:
+
+    msg     The debugging message
+    file    The name of the file where the debug() method was called from
+    line    The line number that it was called from
+    pkg     The package that it was called from
+    class   The class name of the object that the method was called against
+    where   A summary of the package and class
+    date    The current date
+    time    The current time
+
+If the C<class> is the same as the C<pkg> then C<where> will contain the same
+value. If they are different then C<where> will be set equivalent to "<pkg>
+(<class>)". This is the case when the L<debug()> method is called from a base
+class method (C<pkg> will be the base class name from where the call was made)
+against a subclass object (C<class> will be the subclass name). 
+
+See also the L<debug_at()> method which allows you to specify a custom format
+and/or additional placeholder values.
 
 =head2 $MAX_DEPTH
 
@@ -950,7 +1095,7 @@ Andy Wardley L<http://wardley.org/>
 
 =head1 COPYRIGHT
 
-Copyright (C) 1996-2008 Andy Wardley.  All Rights Reserved.
+Copyright (C) 1996-2009 Andy Wardley.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
