@@ -5,6 +5,17 @@
 # DESCRIPTION
 #   Factory module for loading and instantiating other modules.
 #
+# NOTE
+#   This module has grown organically to fit a number of (possibly 
+#   conflicting) needs.  It needs to be completely refactored, and 
+#   probably split into a number of different factory modules.  The 
+#   TT3 code on which this was originally based had separate base class
+#   factory modules for modules (that just got loaded), objects (that 
+#   got loaded and instantiated) and single objects (that got loaded,
+#   created and cached).  With hindsight, it was a mistake to try and 
+#   cram all that functionality into one module.  It should be separated
+#   into a base class module/API and a number of specialised subclasses.
+#
 # AUTHOR
 #   Andy Wardley   <abw@wardley.org>
 #
@@ -12,22 +23,26 @@
 
 package Badger::Factory;
 
+use Badger::Debug ':dump';
 use Badger::Class
     version   => 0.01,
     debug     => 0,
     base      => 'Badger::Prototype Badger::Exporter',
     import    => 'class',
-    utils     => 'plural blessed textlike dotid',
+    utils     => 'plural blessed textlike dotid camel_case',
     words     => 'ITEM ITEMS ISA',
-    constants => 'PKG ARRAY HASH REFS ONCE',
+    constants => 'PKG ARRAY HASH REFS ONCE DEFAULT',
     constant  => {
-        OBJECT       => 'object',
-        FOUND        => 'found',
-        FOUND_REF    => 'found_ref',
-        PATH_SUFFIX  => '_PATH',
+        OBJECT         => 'object',
+        FOUND          => 'found',
+        FOUND_REF      => 'found_ref',
+        PATH_SUFFIX    => '_PATH',
+        NAMES_SUFFIX   => '_NAMES',
+        DEFAULT_SUFFIX => '_DEFAULT',
     },
     messages  => {
         no_item    => 'No item(s) specified for factory to manage',
+        no_default => 'No default defined for %s factory',
         bad_ref    => 'Invalid reference for %s factory item %s: %s',
         bad_method => qq{Can't locate object method "%s" via package "%s" at %s line %s},
     };
@@ -41,7 +56,7 @@ our $AUTOLOAD;
 sub init_factory {
     my ($self, $config) = @_;
     my $class = $self->class;
-    my ($item, $items, $path);
+    my ($item, $items, $path, $map, $default);
 
     # 'item' and 'items' can be specified as config params or we look for
     # $ITEM and $ITEMS variables in the current package or those of any 
@@ -63,17 +78,41 @@ sub init_factory {
     # use 'items' in config, or grokked from $ITEMS, or guess plural
     $items = $config->{ items } || $items || plural($item);
 
-    $path = $config->{ $item.'_path' } || $config->{ path };
-    $path = [ $path ] if $path && ref $path ne ARRAY;
-    $self->{ path   } = $class->list_vars(uc $item . PATH_SUFFIX, $path);
-    $self->{ $items } = $class->hash_vars(uc $items, $config->{ $items });
-    $self->{ items  } = $items;
-    $self->{ item   } = $item;
-    $self->{ loaded } = { };
+    my $ipath    = $item.PATH_SUFFIX;
+    my $inames   = $item.NAMES_SUFFIX;
+    my $idefault = $item.DEFAULT_SUFFIX;
+    
+    # Merge all XXXX_PATH package vars with any 'xxxx_path' or 'path' config 
+    # items.  Ditto for XXXX_NAME / 'xxxx_name' / 'aka' and  XXXXS/ 'xxxxs'
+    
+    my @path  = @$config{ path  => $ipath  };
+    my @names = @$config{ names => lc $inames };
+    $self->{ path     } = $class->list_vars(uc $ipath, @path);
+    $self->{ names    } = $class->hash_vars(uc $inames, @names);
+    $self->{ $items   } = $class->hash_vars(uc $items, $config->{ $items });
+    $self->{ items    } = $items;
+    $self->{ item     } = $item;
+    $self->{ loaded   } = { };
+    $self->{ no_cache } = defined $config->{ no_cache }     # quick hack
+        ? $config->{ no_cache }
+        : $class->any_var('NO_CACHE') || 0;
+
+    # see if a 'xxxx_default' or 'default' configuration option is specified
+    # or look for the first XXXX_DEFAULT or DEFAULT package variable.
+    $default = $config->{ $idefault } 
+            || $config->{ default }
+            || $class->any_var_in( uc $idefault, uc DEFAULT );
+    if ($default) {
+        $self->debug("Setting default to $default") if DEBUG;
+        $self->{ default } = $self->{ names }->{ default } = $default;
+    }
+
+    $self->debug(
+        "Initialised $item/$items factory\n",
+        " Path: ", $self->dump_data($self->{ path }), "\n",
+        "Names: ", $self->dump_data($self->{ names })
+    ) if DEBUG;
 
-    $self->debug("Initialised $item/$items factory") if DEBUG;
-    $self->debug("Path: [", join(', ', @{ $self->{ path } }), "]") if DEBUG;
-        
     return $self;
 }
 
@@ -84,6 +123,13 @@ sub path {
         :  $self->{ path };
 }
 
+sub default {
+    my $self = shift->prototype;
+    return @_ 
+        ? ($self->{ default } = $self->{ names }->{ default } = shift)
+        :  $self->{ default };
+}
+
 sub items {
     my $self  = shift->prototype;
     my $items = $self->{ $self->{ items } };
@@ -95,7 +141,7 @@ sub items {
 }
 
 sub item {
-    my $self = shift->prototype;
+    my $self = shift; $self = $self->prototype unless ref $self;
     my ($type, @args) = $self->type_args(@_);
 
     # In most cases we're expecting $type to be a name (e.g. Table) which we
@@ -121,27 +167,38 @@ sub item {
     my $canon = dotid $type;
 
     $self->debug("Looking for '$type' or '$canon' in $self->{ items }") if DEBUG;
-    
-    my $item  = $items->{ $type  } 
-            ||= $items->{ $canon }
+#   $self->debug("types: ", $self->dump_data($self->{ types })) if DEBUG;
+
+    # false but defined entry indicates the item is not found
+    return $self->not_found($type, \@args)
+        if exists $items->{ $type }
+           && not $items->{ $type };
+
+    my $item = $items->{ $type  } 
+            || $items->{ $canon }
             # TODO: this needs to be defined-or, like //
             # Plugins can return an empty string to indicate that they 
             # do nothing.
             # HMMM.... or does it?
             ||  $self->find($type, \@args)
-            ||  $self->default($type, \@args)
+#            ||  $self->default($type, \@args)
             ||  return $self->not_found($type, \@args);
 
+    $items->{ $type } = $item
+        unless $self->{ no_cache };
+
     return $self->found($type, $item, \@args);
 }
 
 sub type_args {
-    # Simple pass-through method to grok $type and @args from argument list
-    # Subclasses can re-define this to insert their own type mapping or 
+    # Simple method to grok $type and @args from argument list.  The only
+    # processing it does is to set $type to 'default' if it is undefined or
+    # false. Subclasses can re-define this to insert their own type mapping or 
     # argument munging, e.g. to inject values into the configuration params 
     # for an object
     shift;
-    return @_;
+    my $type = shift || DEFAULT;
+    return ($type, @_);
 }
 
 sub find {
@@ -150,6 +207,10 @@ sub find {
     my $bases  = $self->path;
     my $module;
     
+    # run the type through the type map to handle any unusual capitalisation,
+    # spelling, aliases, etc.
+    $type = $self->{ names }->{ $type } || $type;
+    
     foreach my $base (@$bases) {
         return $module
             if $module = $self->load( $self->module_names($base, $type) );
@@ -175,7 +236,7 @@ sub load {
         }
                         
         no strict REFS;
-        $self->debug("attempting to load $module\n") if DEBUG;
+        $self->debug("attempting to load $module") if DEBUG;
 
         # Some filesystems are case-insensitive (like Apple's HFS), so an 
         # attempt to load Badger::Example::foo may succeed, when the correct 
@@ -184,22 +245,20 @@ sub load {
         # loading something that has no ISA.  Need to cross-check with 
         # what's going on in Badger::Class _autoload()
 
-        if ( ( $loaded->{ $module } = class($module)->maybe_load )
+        my $loadname;
+        if ( ( $loadname = class($module)->maybe_load )
         &&   ( ${ $module.PKG.VERSION } || @{ $module.PKG.ISA }  ) ) {
-            $self->debug("loaded $module\n") if DEBUG;
+            $self->debug("loaded $module") if DEBUG;
+            $loaded->{ $module } = $loadname;
             return $module 
         }
 
-        $self->debug("failed to load $module\n") if DEBUG;
+        $self->debug("failed to load $module") if DEBUG;
     }
 
     return undef;
 }
 
-sub default {
-    # No default, by default.  Subclasses can do something here
-    return undef;
-}
 
 sub found {
     my ($self, $type, $item, $args) = @_;
@@ -259,8 +318,11 @@ sub found_array {
 }
 
 sub not_found {
-    my ($self, $name, @args) = @_;
-    $self->error_msg( not_found => $self->{ item }, $name );
+    my ($self, $type, @args) = @_;
+
+    return $type eq DEFAULT
+        ? $self->error_msg( no_default => $self->{ item } )
+        : $self->error_msg( not_found => $self->{ item }, $type );
 }
 
 sub construct {
@@ -271,7 +333,9 @@ sub construct {
 
 sub module_names {
     my $self = shift;
-    my @bits = map { split /\.+/ } @_;
+    my @bits = 
+        map { camel_case($_) }
+        map { split /[\.]+/ } @_;
 
     return (
         join( PKG, map { ucfirst $_ } @bits ),
@@ -279,6 +343,7 @@ sub module_names {
     );
 }
 
+
 sub can {
     my ($self, $name) = @_;
 
@@ -307,7 +372,7 @@ sub AUTOLOAD {
     my ($name) = ($AUTOLOAD =~ /([^:]+)$/ );
     return if $name eq 'DESTROY';
 
-    $self->debug("AUTOLOAD $name\n") if DEBUG;
+    $self->debug("AUTOLOAD $name") if DEBUG;
 
     local $RUNAWAY = $RUNAWAY;
     $self->error("AUTOLOAD went runaway on $name")
@@ -316,7 +381,7 @@ sub AUTOLOAD {
     # upgrade class methods to calls on prototype
     $self = $self->prototype unless ref $self;
 
-    $self->debug("factory item: $self->{ item }\n") if DEBUG;
+    $self->debug("factory item: $self->{ item }") if DEBUG;
     
     if ($name eq $self->{ item }) {
         $self->class->method( $name => $self->can('item') );
@@ -430,9 +495,13 @@ define a factory for them like so:
     package My::Widgets;
     use base 'Badger::Factory';
     
-    our $ITEM        = 'widget';
-    our $ITEMS       = 'widgets';
-    our $WIDGET_PATH = ['My::Widget', 'Your::Widget'];
+    our $ITEM           = 'widget';
+    our $ITEMS          = 'widgets';
+    our $WIDGET_PATH    = ['My::Widget', 'Your::Widget'];
+    our $WIDGET_DEFAULT = 'foo';
+    our $WIDGET_NAMES   = {
+        html => 'HTML',
+    };
 
     # lookup table for any non-standard spellings/capitalisations/paths
     our $WIDGETS     = {
@@ -458,7 +527,20 @@ The C<$WIDGET_PATH> is used to define one or more base module names under
 which your widgets are located.  The name of this variable is derived 
 from the upper case item name in C<$ITEM> with C<_PATH> appended.  In this
 example, the factory will look for the C<Foo::Bar> module as either 
-C<My::Widget::Foo::Bar> or C<Your::Widget::Foo::Bar>.
+C<My::Widget::Foo::Bar> or C<Your::Widget::Foo::Bar>.  
+
+The C<$WIDGET_DEFAULT> specifies the default item name to use if a request
+is made for a module using an undefined or false name.  If you don't specify
+any value for a default then it uses the literal string C<default>.  Adding
+a C<default> entry to your C<$WIDGET_NAMES> or C<$WIDGETS> will have the same
+effect.
+
+The C<$WIDGET_NAMES> is used to define any additional name mappings. This is
+usually required to handle alternate spellings or unusual capitalisations that
+the default name mapping algorithm would get wrong. For example, a request for
+an C<html> widget would look for C<My::Widget::Html> or C<Your::Widget::Html>.
+Adding a C<$WIDGET_MAP> entry mapping C<html> to C<HTML> will instead send it
+looking for C<My::Widget::HTML> or C<Your::Widget::HTML>.
 
 If you've got any widgets that aren't located in one of these locations,
 or if you want to provide some aliases to particular widgets then you can
@@ -492,6 +574,15 @@ constructor method.
 
     my $foo_bar = $widgets->widget('Foo::Bar', x => 10, y => 20);
 
+If you've specified a C<$WIDGET_DEFAULT> for your factory then you can call
+the L<widget()> method without any arguments to get the default object.
+
+    my $widget = $widgets->widget;
+
+You can use the L<default()> method to change the default module.
+
+    $widgets->default('bar');
+
 The factory module can be customised using configuration parameters. For
 example, you can provide additional values for the C<widget_path>, or define
 additional widgets:
@@ -527,6 +618,21 @@ Used to get or set the factory module path.
 
 Calling the method with arguments replaces any existing list.
 
+=head2 names($names)
+
+Used to get or set the names mapping table.
+
+    my $names = $widgets->names;
+    $widgets->names({ html => 'HTML' });
+
+Calling the method with arguments replaces any existing names table.
+
+=head2 default($name)
+
+Used to get or set a name for the default item name.  The default value is
+the literal string C<default>.  This allows you to add a C<default> entry
+to either your L<names()> or L<items()> and it will be located automatically.
+
 =head2 items(%items)
 
 Used to fetch or update the lookup table for mapping names to modules.
@@ -601,13 +707,6 @@ one.  At that point it returns the module name that was successfully
 loaded and ignores the remaining arguments.  If none of the modules can
 be loaded then it returns C<undef>
 
-=head2 default($type,\@args)
-
-This method is called to provide a default value in the case that L<find()>
-wasn't able to find and load a module.  It does nothing in the base class
-factory module, but can be re-defined by subclasses to do something more 
-interesting.
-
 =head2 found($name,$item,\@args)
 
 This method is called when an item has been found, either in the internal
