@@ -17,9 +17,9 @@ use Cwd 'getcwd';
 use Badger::Class
     version   => 0.01,
     debug     => 0,
-    base      => 'Badger::Prototype',
+    base      => 'Badger::Prototype Badger::Filesystem::Base',
     import    => 'class',
-    utils     => 'params is_object',
+    utils     => 'params is_object random_name',
     constants => 'HASH ARRAY TRUE REFS PKG',
     constant  => {
         virtual     => 0,
@@ -31,6 +31,7 @@ use Badger::Class
         UPDIR       =>  File::Spec->updir,
         FS          => 'Badger::Filesystem',
         VFS         => 'Badger::Filesystem::Virtual',
+        UFS         => 'Badger::Filesystem::Universal',
         PATH        => 'Badger::Filesystem::Path',
         FILE        => 'Badger::Filesystem::File',
         DIRECTORY   => 'Badger::Filesystem::Directory',
@@ -39,7 +40,7 @@ use Badger::Class
     exports   => {
         any         => 'FS PATH FILE DIR DIRECTORY cwd getcwd rel2abs abs2rel',
         tags        => { 
-            types   => 'Path File Dir Directory Cwd',
+            types   => 'Path File Dir Directory Cwd Bin',
             dirs    => 'ROOTDIR UPDIR CURDIR',
         },
         hooks       => {
@@ -47,6 +48,10 @@ use Badger::Class
                 # load VFS module and call its export() method
                 class(shift->VFS)->load->pkg->export(shift, shift)
             },
+            UFS     => sub {
+                # load UFS module and call its export() method
+                class(shift->UFS)->load->pkg->export(shift, shift)
+            },
             '$Bin'  => \&_export_findbin_hook,
         },
     },
@@ -55,13 +60,15 @@ use Badger::Class
         delete_failed => 'Failed to delete %s %s: %s',
         bad_volume    => 'Volume mismatch: %s vs %s',
         bad_stat      => 'Nothing known about %s',
+        copy_failed   => 'Failed to %s file from %s to %s: %s',
+        no_path       => 'Unable to determine location of %s',
     };
 
 use Badger::Filesystem::File;
 use Badger::Filesystem::Directory;
 
 #-----------------------------------------------------------------------
-# special export hook to make $Bin available from FindBin
+# special export hooks to make $Bin available from FindBin 
 #-----------------------------------------------------------------------
 
 sub _export_findbin_hook {
@@ -86,6 +93,7 @@ sub _export_findbin_hook {
 *delete_dir   = \&delete_directory;
 *open_dir     = \&open_directory;
 *read_dir     = \&read_directory;
+*temp_dir     = \&temp_directory;
 *dir_child    = \&directory_child;
 *dir_children = \&directory_children;
 *mkdir        = \&create_directory;
@@ -116,10 +124,35 @@ sub _export_findbin_hook {
 # factory subroutines
 #-----------------------------------------------------------------------
 
-sub Path      { return @_ ? FS->path(@_)      : PATH      }
-sub File      { return @_ ? FS->file(@_)      : FILE      }
-sub Directory { return @_ ? FS->directory(@_) : DIRECTORY }
-sub Cwd       { FS->directory }
+sub Path { 
+    return PATH unless @_; 
+    return @_ == 1 && is_object(PATH, $_[0])
+        ? $_[0]                                 # return existing Path object
+        : FS->path(@_);                         # or construct a new one
+}
+
+sub File { 
+    return FILE unless @_; 
+    return @_ == 1 && is_object(FILE, $_[0])
+        ? $_[0]                                 # ditto for File object
+        : FS->file(@_);
+}
+
+sub Directory { 
+    return DIRECTORY unless @_; 
+    return @_ == 1 && is_object(DIRECTORY, $_[0]) 
+        ? $_[0]                                 # ditto for Directory object
+        : FS->directory(@_);
+}
+
+sub Cwd { 
+    FS->directory 
+}
+
+sub Bin { 
+    class(FINDBIN)->load; 
+    FS->directory($FindBin::Bin);
+}
 
 
 #-----------------------------------------------------------------------
@@ -146,9 +179,14 @@ class->methods(
 sub init {
     my ($self, $config) = @_;
 
-    # TODO: have filsystem "styles", e.g. URI/URL style provides defaults
-    # for rootdir, updir, curdir, separator.  But this requires us to bypass
-    # File::Spec, so we'll leave it for now.  KISS.
+    # NEW CODE: trying to abstract out the file specification so that I 
+    # can slot in a Universal file spec decoy which always generates URIs
+    my $spec = $self->{ spec } 
+             = $config->{ spec } 
+            || $config->{ filespec } 
+            || $self->FILESPEC;
+        
+    $self->debug("spec is $spec") if DEBUG;
     
     # The tokens used to represent the root directory ('/'), the 
     # parent directory ('..') and current directory ('.') default to
@@ -156,9 +194,9 @@ sub init {
     # we have to resort to an ugly hack.  The File::Spec module hard-codes 
     # the path separator in the catdir() method so we have to make a round-
     # trip through catdir() to grok the separator in a cross-platform manner
-    $self->{ rootdir   } = $config->{ rootdir   } || ROOTDIR;
-    $self->{ updir     } = $config->{ updir     } || UPDIR;
-    $self->{ curdir    } = $config->{ curdir    } || CURDIR;
+    $self->{ rootdir   } = $config->{ rootdir   } || $spec->rootdir;
+    $self->{ updir     } = $config->{ updir     } || $spec->updir;
+    $self->{ curdir    } = $config->{ curdir    } || $spec->curdir;
     $self->{ separator } = $config->{ separator } || do {
         my $sep = FILESPEC->catdir(('badger') x 2);
         $sep =~ s/badger//g;
@@ -172,9 +210,18 @@ sub init {
     # undefined and let cwd() call getcwd() determine it dynamically
     $self->{ cwd } = $config->{ cwd };
     
+    # additional options, e.g. codec, encoding
+    $self->init_options($config);
+    
     return $self;
 }
 
+sub spec {
+    return ref $_[0] eq HASH
+        ? $_[0]->{ spec }
+        : FILESPEC;
+}
+
 sub path {
     Path->new( shift->_child_args( path => @_ ) );
 }
@@ -225,8 +272,9 @@ sub cwd {
 
 sub merge_paths {
     my ($self, $base, $path) = @_;
-    my @p1 = FILESPEC->splitpath($base);
-    my @p2 = FILESPEC->splitpath($path);
+    my $spec = $self->spec;
+    my @p1   = $spec->splitpath($base);
+    my @p2   = $spec->splitpath($path);
 
     # check volumes match
     if (defined $p2[0]) {
@@ -238,28 +286,30 @@ sub merge_paths {
     my $vol = shift(@p1) || '';
     my $file = pop @p2;
     
-    FILESPEC->catpath($vol, FILESPEC->catdir(@p1, @p2), $file);
+    $spec->catpath($vol, $spec->catdir(@p1, @p2), $file);
 }
     
 sub join_path {
     my $self = shift;
     my @args = map { defined($_) ? $_ : '' } @_[0..2];
-    FILESPEC->canonpath( FILESPEC->catpath(@args) );
+    my $spec = $self->spec;
+    $spec->canonpath( $spec->catpath(@args) );
 }
 
 sub join_directory {
     my $self = shift;
     my $dir  = @_ == 1 ? shift : [ @_ ];
+    my $spec = $self->spec;
     $self->debug("join_dir(", ref $dir eq ARRAY ? '[' . join(', ', @$dir) . ']' : $dir, ")\n") if $DEBUG;
     ref $dir eq ARRAY
-        ? FILESPEC->catdir(@$dir)
-        : FILESPEC->canonpath($dir);
+        ? $spec->catdir(@$dir)
+        : $spec->canonpath($dir);
 }
 
 sub split_path {
     my $self  = shift;
     my $path  = $self->join_directory(@_);
-    my @split = map { defined($_) ? $_ : '' } FILESPEC->splitpath($path);
+    my @split = map { defined($_) ? $_ : '' } $self->spec->splitpath($path);
     $self->debug("split_path($path) => ", join(', ', @split), "\n") if $DEBUG;
     return wantarray ? @split : \@split;
 }
@@ -267,7 +317,7 @@ sub split_path {
 sub split_directory {
     my $self  = shift;
     my $path  = $self->join_directory(@_);
-    my @split = FILESPEC->splitdir($path);
+    my @split = $self->spec->splitdir($path);
     return wantarray ? @split : \@split;
 }
 
@@ -309,7 +359,10 @@ sub slash_directory {
 
 sub is_absolute {
     my $self = shift;
-    FILESPEC->file_name_is_absolute($self->join_directory(@_)) ? 1 : 0;
+#    $self->debug("args: ", $self->dump_data(\@_));
+    $self->spec->file_name_is_absolute(
+        $self->join_directory(@_)
+    ) ? 1 : 0;
 }
 
 sub is_relative {
@@ -319,13 +372,14 @@ sub is_relative {
 sub absolute {
     my $self = shift;
     my $path = $self->join_directory(shift);
-    return $path if FILESPEC->file_name_is_absolute($path);
-    FILESPEC->catdir(shift || $self->cwd, $path);
+    my $spec = $self->spec;
+    return $path if $spec->file_name_is_absolute($path);
+    $spec->catdir(shift || $self->cwd, $path);
 }
 
 sub relative {
     my $self = shift;
-    FILESPEC->abs2rel($self->join_directory(shift), shift || $self->cwd);
+    $self->spec->abs2rel($self->join_directory(shift), shift || $self->cwd);
 }
 
 
@@ -334,23 +388,25 @@ sub relative {
 #-----------------------------------------------------------------------
 
 sub path_exists {
-    my $self = shift;
-    return -e $self->definitive_read(shift);
+    shift->stat_path(@_);
 }
 
 sub file_exists {
-    my $self = shift;
-    return -f $self->definitive_read(shift);
+    my $self  = shift;
+    my $stats = $self->stat_path(shift) || return; 
+    return -f _ ? $stats : 0;       # relies on cached stat
 }
 
 sub directory_exists {
     my $self = shift;
-    return -d $self->definitive_read(shift);
+    my $stats = $self->stat_path(shift) || return; 
+    return -d _ ? $stats : 0;       # relies on cached stat
 }
 
 sub stat_path {
     my $self  = shift;
-    my @stats = (stat($self->definitive_read(shift)), -r _, -w _, -x _, -o _);
+    my $path  = $self->definitive_read(shift) || return; 
+    my @stats = (stat($path), -r _, -w _, -x _, -o _, $path);
 
     return $self->error_msg( bad_stat => $self->{ path } )
         unless @stats;
@@ -359,6 +415,12 @@ sub stat_path {
         ?  @stats
         : \@stats;
 }
+
+sub chmod_path {
+    my $self = shift;
+    my $path = $self->definitive_write(shift);
+    chmod(shift, $path);
+}
     
 
 #-----------------------------------------------------------------------
@@ -397,20 +459,29 @@ sub open_file {
     my $self = shift;
     my $name = shift;
     my $mode = $_[0] || 'r';            # leave it in @_ for IO::File
+    my $opts = @_ && ref $_[-1] eq HASH ? pop(@_) : { };
     my $path = $mode eq 'r' 
         ? $self->definitive_read($name)
         : $self->definitive_write($name);
+    return $self->error_msg( no_path => $name )
+        unless defined $path && length $path;
 
     require IO::File;
     $self->debug("about to open file $path (", join(', ', @_), ")\n") if $DEBUG;
 
-    return IO::File->new($path, @_)
+    my $fh = IO::File->new($path, @_)
         || $self->error_msg( open_failed => file => $path => $! );
+
+    $fh->binmode( $opts->{ encoding } ) 
+        if $opts->{ encoding };
+
+    return $fh;
 }
 
 sub read_file {
     my $self = shift;
-    my $fh   = $self->open_file(shift, 'r');
+    my $opts = @_ && ref $_[-1] eq HASH ? pop(@_) : { };
+    my $fh   = $self->open_file(shift, 'r', $opts);
     return wantarray
         ? <$fh>
         : do { local $/ = undef; <$fh> };
@@ -418,7 +489,8 @@ sub read_file {
 
 sub write_file {
     my $self = shift;
-    my $fh   = $self->open_file(shift, 'w');
+    my $opts = @_ && ref $_[-1] eq HASH ? pop(@_) : { };
+    my $fh   = $self->open_file(shift, 'w', $opts);
     return $fh unless @_;           # return handle if no args
     print $fh @_;                   # or print args and close
     $fh->close;
@@ -427,7 +499,8 @@ sub write_file {
 
 sub append_file {
     my $self = shift;
-    my $fh   = $self->open_file(shift, 'a');
+    my $opts = @_ && ref $_[-1] eq HASH ? pop(@_) : { };
+    my $fh   = $self->open_file(shift, 'a', $opts);
     return $fh unless @_;           # return handle if no args
     print $fh @_;                   # or print args and close
     $fh->close;
@@ -435,19 +508,73 @@ sub append_file {
 }
 
 
+sub copy_file {
+    shift->_file_copy( copy => @_ );
+}
+
+sub move_file {
+    shift->_file_copy( move => @_ );
+}
+
+sub _file_copy {
+    require File::Copy;
+
+    my ($self, $action, $from, $to, $params) 
+     = (shift, shift, shift, shift, params(@_));
+     
+    my $src    
+        = is_object(PATH, $from)    ? $from->definitive     # path object
+        : ref($from)                ? $from                 # file handle
+        : $self->definitive_read($from);                    # file path
+
+    my $dest
+        = is_object(PATH, $to)      ? $to->definitive       # as above
+        : ref($to)                  ? $to            
+        : $self->definitive_write($to);
+    
+    my $code 
+        = $action eq 'copy' ? \&File::Copy::copy
+        : $action eq 'move' ? \&File::Copy::move
+        : return $self->error( invalid => action => $action );
+
+    my $file;
+
+    unless (ref $dest) {
+        # NOTE: don't use $self->file($dest) because $self could be a 
+        # VFS and $dest is already a definitive path
+        $file = File($dest);
+        $file->directory->must_exist(
+            $params->{ mkdir    },
+            $params->{ dir_mode },
+        );
+    }
+
+    $code->($src, $dest)
+        || return $self->error_msg( copy_failed => $action, $from, $to, $! );
+
+    my $mode = $params->{ file_mode };
+       $mode = $params->{ mode } unless defined $mode;
+
+    $file->chmod($mode) 
+        if $file && defined $mode;
+
+    return $file || $dest;
+}
+
+
 #-----------------------------------------------------------------------
 # directory manipulation methods
 #-----------------------------------------------------------------------
 
 sub create_directory { 
-    my $self = shift;
-    my $path = $self->definitive_write(shift);
+    my $self   = shift;
+    my $path   = $self->definitive_write(shift);
 
     require File::Path;
 
     eval { 
         local $Carp::CarpLevel = 1;
-        File::Path::mkpath($path, @_) 
+        File::Path::mkpath($path, 0, @_) 
     } || return $self->error($@);
 } 
     
@@ -478,7 +605,7 @@ sub read_directory {
     while (defined ($path = $dirh->read)) {
         push(@paths, $path);
     }
-    @paths = FILESPEC->no_upwards(@paths)
+    @paths = $self->spec->no_upwards(@paths)
         unless $all || ref $self && $self->{ all_entries };
 
     $dirh->close;
@@ -503,6 +630,26 @@ sub directory_children {
     return wantarray ? @paths : \@paths;
 }
 
+
+#-----------------------------------------------------------------------
+# temporary directory/file methods
+#-----------------------------------------------------------------------
+
+sub temp_directory {
+    my $self = shift;
+    return $self->directory( FILESPEC->tmpdir, @_ )->must_exist(1);
+}
+
+sub temp_file {
+    my $self = shift;
+    return $self->temp_directory->file( @_ ? @_ : random_name() )
+}
+
+
+#-----------------------------------------------------------------------
+# visitor methods
+#-----------------------------------------------------------------------
+
 sub visitor {
     my $self  = shift;
     my $vtype = $self->VISITOR;
@@ -522,7 +669,7 @@ sub collect {
 }
 
 sub accept {
-    $_[0]->root->accept($_[1]);
+    shift->root->accept(@_);
 }
 
 #-----------------------------------------------------------------------
@@ -532,26 +679,28 @@ sub accept {
 sub _child_args {
     my $self = shift->prototype;
     my $type = shift;
-    my $args;
-    
-    if (! @_) {
-        $args = { path => undef };
+    my $args = { %{ $self->{ options } } };
+
+    if (@_ && ref $_[-1] eq HASH) {
+        my $more = pop @_;
+        @$args{ keys %$more } = values %$more;
+    }
+
+    if (@_ > 1) {
+        $args->{ path } = [@_];
     }
     elsif (@_ == 1) {
-        $args = ref $_[0] eq HASH ? shift : { path => shift };
-#            : ! ref $_[0] ? { path => shift }
-#            : return $self->error_msg( unexpected => "$type arguments" => $_[0], 'hash ref' )
+        $args->{ path } = shift;
     }
     else {
-        $args = { path => [@_] };
+        $args->{ path } = undef;
     }
+
     $args->{ filesystem } = $self;
     return $args;
 }
 
 
-# TODO: move file?
-
 
 1;
 
@@ -841,12 +990,23 @@ path as a single string or list of path components.
 This returns a L<Badger::Filesystem::Directory> object for the current
 working directory.
 
-    use Badger::Filesystem Cwd;
+    use Badger::Filesystem 'Cwd';
     
     print Cwd;              # /foraging/for/nuts/and/berries
     print Cwd->parent;      # /foraging/for/nuts/and
 
-=head2 cwd
+=head2 Bin()
+
+This returns a L<Badger::Filesystem::Directory> object for the directory
+in which the currently executing script is located.  It is a simple 
+wrapper around the value defined in L<$Bin>.
+
+    use Badger::Filesystem 'Bin';
+    
+    print Bin;              # /path/to/current/script
+    print Bin->parent;      # /path/to/current
+
+=head2 cwd()
 
 This returns a simple text string representing the current working directory.
 It is a a wrapper around the C<getcwd> function in L<Cwd>.  It also 
@@ -867,17 +1027,19 @@ This is exactly the same as:
     use FindBin '$Bin';
     use lib "$Bin/../lib";
 
-The benefit is that you can use it in conjunction with other import options.
+One benefit is that you can use it in conjunction with other import options
+to save on a little typing.  For example:
 
-    use Badger::Filesystem '$Bin Cwd File';
+    use Badger::Filesystem 'Cwd File $Bin';
 
 Compared to something like:
 
-    use FindBin '$Bin';
     use Cwd;
     use Path::Class;
+    use FindBin '$Bin';
+    use lib "$Bin/../lib";
 
-=head2 getcwd
+=head2 getcwd()
 
 This is a direct alias to the C<getcwd> function in L<Cwd>.
 
@@ -1000,7 +1162,7 @@ name into a single path.  This is a wrapper around the
 L<catpath()|File::Spec/catpath()> and L<canonpath()|File::Spec/canonpath()> 
 functions in L<File::Spec>.
 
-    my $path = $fs−>join_path($volume, $directory, $file);
+    my $path = $fs->join_path($volume, $directory, $file);
 
 =head2 split_dir($dir) / split_directory($dir)
 
@@ -1014,11 +1176,11 @@ around the L<splitdir()|File::Spec/splitdir()> function in L<File::Spec>.
 Combines multiple directory names into a single path.  This is a wrapper
 around the L<catdir()|File::Spec/catdir()> function in L<File::Spec>.
 
-    my $dir = $fs−>join_dir('path', 'to', 'my', 'dir');
+    my $dir = $fs->join_dir('path', 'to', 'my', 'dir');
 
 The final element can also be a file name.   TODO: is that portable?
 
-    my $dir = $fs−>join_dir('path', 'to', 'my', 'file');
+    my $dir = $fs->join_dir('path', 'to', 'my', 'file');
 
 =head2 collapse_dir($dir) / collapse_directory($dir)
 
@@ -1183,6 +1345,12 @@ L<Badger::Filesystem::Path> for further details.
      15     file is executable by current process
      16     file is owned by current process
 
+=head2 chmod_path($path)
+
+Changes the file permissions on a path.
+
+    $fs->chmod_path('/path/to/file', 0755);
+
 =head1 FILE MANIPULATION METHODS
 
 =head2 create_file($path)
@@ -1254,6 +1422,37 @@ to indicate success.  Errors are thrown as exceptions.
 
     $fs->append_file('/path/to/file', "Hello World\n", "Regards, Badger\n");
 
+=head2 copy_file($from, $to, %params)
+
+Copies a file from the C<$from> path to the C<$to> path, using L<File::Copy>
+
+    $fs->copy_file($from, $to);
+
+The C<$from> and C<$to> arguments can be file names, file objects, or file
+handles.
+
+An optional list or reference to a hash array of named parameters can follow
+the file names.  The C<mkdir> option can be set to indicate that 
+the destination direction should be created if it doesn't already exist,
+along with any intermediate directories.  
+
+    $fs->copy_file($from, $to, mkdir => 1);
+
+The C<dir_mode> parameter can be used to specify the octal file 
+permissions for any directories created.
+
+    $fs->copy_file($from, $to, 1, mkdir => 1, dir_mode => 0770);
+
+The C<file_mode> parameter (or C<mode> for short) can be used to specify the
+octal file permissions for the created file.
+
+    $fs->copy_file($from, $to, file_mode => 0644);
+
+=head2 move_file($from, $to, %params)
+
+Moves a file from the C<$from> path to the C<$to> path, using L<File::Copy>.
+The arguments are as per L<copy_file()>.
+
 =head1 DIRECTORY MANIPULATION METHODS
 
 =head2 create_dir($path) / create_directory($path) / mkdir($path)
@@ -1313,6 +1512,28 @@ returned as L<Badger::Filesystem::File> objects, directories as
 L<Badger::Filesystem::File> objects. Anything else is returned as a generic
 L<Badger::Filesystem::Path> object.
 
+=head1 TEMPORARY DIRECTORY AND FILE METHODS
+
+=head2 temp_dir($dir) / temp_directory($dir)
+
+This returns a reference to a L<Badger::Filesystem::Directory> object for the
+temporary directory on your system (as reported by C<tmpdir> in L<File::Spec>).
+
+    my $tmp = $fs->temp_dir;
+
+If any arguments are specified then they are appended as sub-directories to
+the temporary directory path.
+
+    my $tmp = $fs->temp_dir('foo', 'bar');  # e.g. /tmp/foo/bar 
+
+=head2 temp_file($name)
+
+This returns a reference to a L<Badger::Filesystem::File> object for a named
+file created in the temporary directory returned by the L<temp_directory()>
+method.
+
+    my $file = $fs->temp_file('foo.tmp');   # e.g. /tmp/foo.tmp
+
 =head1 VISITOR METHODS
 
 =head2 visitor(\%params)
@@ -1423,11 +1644,14 @@ An alias for C<Badger::Filesystem::Directory>
 
 =head1 AUTHOR
 
-Andy Wardley E<lt>abw@wardley.orgE<gt>
+Andy Wardley L<http://wardley.org/>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2005-2008 Andy Wardley. All rights reserved.
+Copyright (C) 2005-2009 Andy Wardley. All rights reserved.
+
+This module is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself.
 
 =head1 ACKNOWLEDGEMENTS
 
